# Reviewer

Reviewer automates reviews of Drupal configuration to help developers catch 
build issues and decrease the time spent checking configuration forms.

For a full description of the module, visit the
[project page](https://www.drupal.org/project/reviewer).

Submit bug reports and feature suggestions, or track changes in the
[issue queue](https://www.drupal.org/project/issues/search/reviewer).

## Installation

Install as you would normally install a contributed Drupal module. For further
information, see
[Installing Drupal Modules](https://www.drupal.org/docs/extending-drupal/installing-drupal-modules).

## Configuration

Reviewer requires no configuration. It is recommended to enable reviewer on
development environments only using
[Configuration Split](https://www.drupal.org/project/config_split).

## Usage

Reviews can be run through Drush or reviewed at `/admin/reports/reviewer` when
the `reviewer_ui` submodule is enabled.

Reviewer provides the following Drush commands:

- `reviewer:list` 
- `reviewer:check`
- `reviewer:ignore`
- `reviewer:fix`

Use the `--help` flag to see available options for each command

Enable the `reviewer_examples` submodule to enable some example reviews to
try out Reviewer

### reviewer:list

Lists all available reviews.

### reviewer:check

Runs reviews, checking for issues. By default, running reviews will output all
failures (tests which did not pass) and errors (tests which threw an exception).

You may specify specific reviews to run by their ID. For example, 
`reviewer:check test_node test_taxonomy` runs the "Test Node" and "Test
Taxonomy" reviews. For reviews that support bundles, you can specify specific
bundles to run. `reviewer:run test_node:test_fail` runs the "Test Node" review
on the "test_fail" content type only.

#### Options

- `show-ignored`: Display passed tests in addition to failures and errors. 
- `show-passed`: Display tests which were not run in addition to failures and 
  errors. 
- `show-not-run`: Display ignored tests in addition to failures and errors. 
- `show-all`: Display ignored and passed tests in addition to failures and
  errors.

### reviewer:fix

Fixes failures found by reviews. Failures marked as ignored will not be fixed.
You may specify reviews and bundles in the same manner as `reviewer:check`.

### reviewer:ignore

Ignores failures and errors found by reviews. You may specify reviews and 
bundles in the same manner as `reviewer:check`.

#### Options

- `existing`: Review and update existing ignored failures and errors as well as 
  unignored ones.
- `reset`: Unignore previously ignored failures and errors before ignoring.
- `unignore`: Unignore previously ignored failures and errors.

## Creating Reviews, Checklists, and Tasks

Reviews are plugins which represent a logical grouping of items to review. For
example, a `node` review which checks configuration of all node bundles, or a
`site_settings` review which checks various `system.*` configuration.

Each review must reference at least one checklist plugin. Checklists are 
smaller groupings of tasks which can be reused across reviews. For example, a 
`search_index` checklist which ensures that the search index display mode is
configured and that it has all field labels set to hidden. The `search_index`
checklist is then added to the `node`, `taxonomy_term`, and `paragraph` reviews,
checking that search index view modes are configured correctly across the three
entity types.

Each checklist must reference at least on task plugin. Tasks are the individual
items run to check configuration values. Every task must implement the
`\Drupal\reviewer\Reviewer\Task\TaskInterface::check` method which is executed
by Reviewer to check values. Tasks may also declare their ability to fix values
automatically by implementing
`\Drupal\reviewer\Reviewer\Task\FixableInterface::fix`.

### Reviews

Create review plugins in the `\Drupal\my_module\Plugin\reviewer\Review`
namespace. The `\Drupal\reviewer\Attribute\Review` attribute is used to
configure reviews. All reviews must extend the
`\Drupal\reviewer\Plugin\reviewer\Review\ReviewBase` base class.

Here is a review that runs a site configuration checklist which contains tasks
to check the `system.*` configuration values:

```php
#[Review(
  id: 'site_configuration',
  label: 'Site Configuration Review',
  checklists: [
    SiteConfigurationChecklist::class,
  ],
)]
final class SiteConfigurationReview extends ReviewBase {}
```

### Checklists

Create checklist plugins in the `\Drupal\my_module\Plugin\reviewer\Checklist`
namespace. The `\Drupal\reviewer\Attribute\Checklist` attribute is used to
configure checklists. All checklists must extend the 
`\Drupal\reviewer\Reviewer\Checklist\ChecklistBase` base class.

Here is the site configuration checklist from the review above, which contains
tasks to check various `system.*` configuration values:

```php
#[Checklist(
  id: 'site_configuration',
  tasks: [
    SystemPerformance::class,
    SystemSite::class,
  ],
)]
final class SiteConfigurationChecklist extends ChecklistBase {}
```

### Tasks

Tasks are where the checking, and optional fixing, logic is defined. Create 
task plugins in the `\Drupal\my_module\Plugin\reviewer\Task` namespace. The
`\Drupal\reviewer\Attribute\Task` attribute is used to configure checklists. 
All checklists must extend the `\Drupal\reviewer\Reviewer\Task\TaskBase` base
class, although there are better classes for extending tasks found in the
`reviewer_test_kit` submodule which can greatly speed up creating tasks. 

Here are the system performance tasks from the checklist above:

```php
#[Task('system_performance')]
final class SystemPerformance extends ConfigTaskBase {

  /**
   * {@inheritdoc}
   */
  protected string $configName = 'system.performance';

  /**
   * {@inheritdoc}
   */
  protected array $expectedValues = [
    'cache' => [
      'page' => [
        'max_age' => 86400,
      ],
    ],
    // Ensure that assets are combined and compressed.
    'css' => [
      'preprocess' => TRUE,
      'gzip' => TRUE,
    ],
    'js' => [
      'preprocess' => TRUE,
      'gzip' => TRUE,
    ],
  ];

}
```

## Reviewer Test Kit

This submodule contains base tasks for the streamlined creating of tasks for
checking and fixing a variety of configuration types and values:

- Simple Configuration: [ConfigTaskBase](https://git.drupalcode.org/project/reviewer/-/blob/2.x/modules/reviewer_test_kit/src/Plugin/reviewer/Task/ConfigTaskBase.php)
- Entity Form Displays
  - Disabling Form Fields: [FormFieldsDisabledTaskBase](https://git.drupalcode.org/project/reviewer/-/blob/2.x/modules/reviewer_test_kit/src/Plugin/reviewer/Task/Entity/Display/Form/FormFieldsDisabledTaskBase.php)
  - Form Field Types: [FieldWidgetsTaskBase](https://git.drupalcode.org/project/reviewer/-/blob/2.x/modules/reviewer_test_kit/src/Plugin/reviewer/Task/Entity/Display/Form/FieldWidgetsTaskBase.php)
  - Field Widget Settings: [FieldWidgetSettingsTaskBase](https://git.drupalcode.org/project/reviewer/-/blob/2.x/modules/reviewer_test_kit/src/Plugin/reviewer/Task/Entity/Display/Form/FieldWidgetSettingsTaskBase.php)
  - Field Group Settings: [FormFieldGroupSettingsTaskBase](https://git.drupalcode.org/project/reviewer/-/blob/2.x/modules/reviewer_test_kit/src/Plugin/reviewer/Task/Entity/Display/Form/FormFieldGroupSettingsTaskBase.php)
- Entity View Displays
  - Disabling Fields: [ViewFieldsDisabledTaskBase](https://git.drupalcode.org/project/reviewer/-/blob/2.x/modules/reviewer_test_kit/src/Plugin/reviewer/Task/Entity/Display/View/ViewFieldsDisabledTaskBase.php)
  - Field Formatter Types: [FieldFormattersTaskBase](https://git.drupalcode.org/project/reviewer/-/blob/2.x/modules/reviewer_test_kit/src/Plugin/reviewer/Task/Entity/Display/View/FieldFormattersTaskBase.php)
  - Field Formatter Settings: [FieldFormatterSettingsTaskBase](https://git.drupalcode.org/project/reviewer/-/blob/2.x/modules/reviewer_test_kit/src/Plugin/reviewer/Task/Entity/Display/View/FieldFormatterSettingsTaskBase.php)
  - Field Group Settings: [ViewFieldGroupSettingsTaskBase](https://git.drupalcode.org/project/reviewer/-/blob/2.x/modules/reviewer_test_kit/src/Plugin/reviewer/Task/Entity/Display/View/ViewFieldGroupSettingsTaskBase.php)
- Entity Types: [EntityTypeTaskBase](https://git.drupalcode.org/project/reviewer/-/blob/2.x/modules/reviewer_test_kit/src/Plugin/reviewer/Task/Entity/EntityTypeTaskBase.php)

## Maintainers

Current maintainer:
- [Benjamin Baird (benabaird)](https://www.drupal.org/u/benabaird)
